@@ -1,7 +1,7 @@
 /* indices.c -- deal with an Info file index.
-   $Id: indices.c,v 1.9 2007/11/25 19:25:34 karl Exp $
+   $Id: indices.c,v 1.21 2012/11/30 23:58:20 gray Exp $
 
-   Copyright (C) 1993, 1997, 1998, 1999, 2002, 2003, 2004, 2007
+   Copyright (C) 1993, 1997, 1998, 1999, 2002, 2003, 2004, 2007, 2008, 2011
    Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
@@ -21,23 +21,26 @@
 
 #include "info.h"
 #include "indices.h"
+#include "variables.h"
 
 /* User-visible variable controls the output of info-index-next. */
 int show_index_match = 1;
 
 /* In the Info sense, an index is a menu.  This variable holds the last
    parsed index. */
-static REFERENCE **index_index = (REFERENCE **)NULL;
+static REFERENCE **index_index = NULL;
 
 /* The offset of the most recently selected index element. */
 static int index_offset = 0;
+/* Whether we are doing partial index search */
+static int index_partial = 0;
 
 /* Variable which holds the last string searched for. */
-static char *index_search = (char *)NULL;
+static char *index_search = NULL;
 
 /* A couple of "globals" describing where the initial index was found. */
-static char *initial_index_filename = (char *)NULL;
-static char *initial_index_nodename = (char *)NULL;
+static char *initial_index_filename = NULL;
+static char *initial_index_nodename = NULL;
 
 /* A structure associating index names with index offset ranges. */
 typedef struct {
@@ -47,7 +50,7 @@ typedef struct {
 } INDEX_NAME_ASSOC;
 
 /* An array associating index nodenames with index offset ranges. */
-static INDEX_NAME_ASSOC **index_nodenames = (INDEX_NAME_ASSOC **)NULL;
+static INDEX_NAME_ASSOC **index_nodenames = NULL;
 static int index_nodenames_index = 0;
 static int index_nodenames_slots = 0;
 
@@ -60,7 +63,7 @@ add_index_to_index_nodenames (REFERENCE **array, NODE *node)
   INDEX_NAME_ASSOC *assoc;
 
   for (last = 0; array[last + 1]; last++);
-  assoc = (INDEX_NAME_ASSOC *)xmalloc (sizeof (INDEX_NAME_ASSOC));
+  assoc = xmalloc (sizeof (INDEX_NAME_ASSOC));
   assoc->name = xstrdup (node->nodename);
 
   if (!index_nodenames_index)
@@ -91,24 +94,24 @@ info_indices_of_window (WINDOW *window)
 
   fb = file_buffer_of_window (window);
 
-  return (info_indices_of_file_buffer (fb));
+  return info_indices_of_file_buffer (fb);
 }
 
 REFERENCE **
 info_indices_of_file_buffer (FILE_BUFFER *file_buffer)
 {
   register int i;
-  REFERENCE **result = (REFERENCE **)NULL;
+  REFERENCE **result = NULL;
 
   /* No file buffer, no indices. */
   if (!file_buffer)
-    return ((REFERENCE **)NULL);
+    return NULL;
 
   /* Reset globals describing where the index was found. */
   maybe_free (initial_index_filename);
   maybe_free (initial_index_nodename);
-  initial_index_filename = (char *)NULL;
-  initial_index_nodename = (char *)NULL;
+  initial_index_filename = NULL;
+  initial_index_nodename = NULL;
 
   if (index_nodenames)
     {
@@ -119,7 +122,7 @@ info_indices_of_file_buffer (FILE_BUFFER *file_buffer)
         }
 
       index_nodenames_index = 0;
-      index_nodenames[0] = (INDEX_NAME_ASSOC *)NULL;
+      index_nodenames[0] = NULL;
     }
 
   /* Grovel the names of the nodes found in this file. */
@@ -135,7 +138,8 @@ info_indices_of_file_buffer (FILE_BUFFER *file_buffer)
               REFERENCE **menu;
 
               /* Found one.  Get its menu. */
-              node = info_get_node (tag->filename, tag->nodename);
+              node = info_get_node (tag->filename, tag->nodename, 
+                                    PARSE_NODE_VERBATIM);
               if (!node)
                 continue;
 
@@ -164,7 +168,7 @@ info_indices_of_file_buffer (FILE_BUFFER *file_buffer)
     if (!result[i]->filename)
       result[i]->filename = xstrdup (file_buffer->filename);
 
-  return (result);
+  return result;
 }
 
 DECLARE_INFO_COMMAND (info_index_search,
@@ -186,24 +190,24 @@ do_info_index_search (WINDOW *window, int count, char *search_string)
 
   /* The user is selecting a new search string, so flush the old one. */
   maybe_free (index_search);
-  index_search = (char *)NULL;
+  index_search = NULL;
 
   /* If this window's file is not the same as the one that we last built an
      index for, build and remember an index now. */
   fb = file_buffer_of_window (window);
   if (!initial_index_filename ||
+      !fb ||
       (FILENAME_CMP (initial_index_filename, fb->filename) != 0))
     {
       info_free_references (index_index);
-      window_message_in_echo_area ((char *) _("Finding index entries..."),
-          NULL, NULL);
+      window_message_in_echo_area (_("Finding index entries..."));
       index_index = info_indices_of_file_buffer (fb);
     }
 
   /* If there is no index, quit now. */
   if (!index_index)
     {
-      info_error ((char *) _("No indices found."), NULL, NULL);
+      info_error (_("No indices found."));
       return;
     }
 
@@ -213,7 +217,7 @@ do_info_index_search (WINDOW *window, int count, char *search_string)
     line = xstrdup (search_string);
   else
     {
-      line = info_read_maybe_completing (window, (char *) _("Index entry: "),
+      line = info_read_maybe_completing (window, _("Index entry: "),
                                          index_index);
       window = active_window;
 
@@ -234,7 +238,8 @@ do_info_index_search (WINDOW *window, int count, char *search_string)
               NODE *node;
 
               node = info_get_node (initial_index_filename,
-                                    initial_index_nodename);
+                                    initial_index_nodename,
+                                    PARSE_NODE_DFLT);
               set_remembered_pagetop_and_point (window);
               window_set_node_of_window (window, node);
               remember_window_and_node (window, node);
@@ -244,6 +249,14 @@ do_info_index_search (WINDOW *window, int count, char *search_string)
         }
     }
 
+  if (mbslen (line) < min_search_length)
+    {
+      info_error (_("Search string too short"));
+      free (line);
+      return;
+    }
+
+  
   /* The user typed either a completed index label, or a partial string.
      Find an exact match, or, failing that, the first index entry containing
      the partial string.  So, we just call info_next_index_match () with minor
@@ -259,8 +272,11 @@ do_info_index_search (WINDOW *window, int count, char *search_string)
         index_offset = i;
       }
     else
-      index_offset = -1;
-
+      {
+	index_offset = -1;
+	index_partial = 0;
+      }
+    
     old_offset = index_offset;
 
     /* The "last" string searched for is this one. */
@@ -287,8 +303,9 @@ index_entry_exists (WINDOW *window, char *string)
     return 0;
 
   fb = file_buffer_of_window (window);
-  if (!initial_index_filename
-      || (FILENAME_CMP (initial_index_filename, fb->filename) != 0))
+  if (!initial_index_filename ||
+      !fb ||
+      (FILENAME_CMP (initial_index_filename, fb->filename) != 0))
     {
       info_free_references (index_index);
       index_index = info_indices_of_file_buffer (fb);
@@ -320,25 +337,51 @@ index_entry_exists (WINDOW *window, char *string)
   return 1;
 }
 
+/* Return true if ENT->label matches "S( <[0-9]+>)?", where S stands
+   for the first LEN characters from STR. */
+static int
+index_entry_matches (REFERENCE *ent, const char *str, size_t len)
+{
+  char *p;
+  
+  if (strncmp (ent->label, str, len))
+    return 0;
+  p = ent->label + len;
+  if (!*p)
+    return 1;
+  if (p[0] == ' ' && p[1] == '<')
+    {
+      for (p += 2; *p; p++)
+	{
+	  if (p[0] == '>' && p[1] == 0)
+	    return 1;
+	  else if (!isdigit (*p))
+	    return 0;
+	}
+    }
+  return 0;
+}
+
 DECLARE_INFO_COMMAND (info_next_index_match,
  _("Go to the next matching index item from the last `\\[index-search]' command"))
 {
   register int i;
   int partial, dir;
   NODE *node;
-
+  size_t search_len;
+  
   /* If there is no previous search string, the user hasn't built an index
      yet. */
   if (!index_search)
     {
-      info_error ((char *) _("No previous index search string."), NULL, NULL);
+      info_error (_("No previous index search string."));
       return;
     }
 
   /* If there is no index, that is an error. */
   if (!index_index)
     {
-      info_error ((char *) _("No index entries."), NULL, NULL);
+      info_error (_("No index entries."));
       return;
     }
 
@@ -349,30 +392,47 @@ DECLARE_INFO_COMMAND (info_next_index_match,
   else
     dir = 1;
 
-  /* Search for the next occurence of index_search.  First try to find
-     an exact match. */
+  /* Search for the next occurence of index_search. */
   partial = 0;
+  search_len = strlen (index_search);
 
-  for (i = index_offset + dir; (i > -1) && (index_index[i]); i += dir)
-    if (strcmp (index_search, index_index[i]->label) == 0)
-      break;
-
-  /* If that failed, look for the next substring match. */
-  if ((i < 0) || (!index_index[i]))
+  if (!index_partial)
     {
+      /* First try to find an exact match. */
       for (i = index_offset + dir; (i > -1) && (index_index[i]); i += dir)
-        if (string_in_line (index_search, index_index[i]->label) != -1)
-          break;
+	if (index_entry_matches (index_index[i], index_search, search_len))
+	  break;
 
-      if ((i > -1) && (index_index[i]))
-        partial = string_in_line (index_search, index_index[i]->label);
+      /* If that failed, look for the next substring match. */
+      if ((i < 0) || (!index_index[i]))
+	{
+	  index_offset = 0;
+	  index_partial = 1;
+	}
     }
 
+  if (index_partial)
+    {
+      /* When looking for substrings, take care not to return previous exact
+	 matches. */
+      for (i = index_offset + dir; (i > -1) && (index_index[i]); i += dir)
+        if (!index_entry_matches (index_index[i], index_search, search_len) &&
+	    string_in_line (index_search, index_index[i]->label) != -1)
+	  {
+	    partial = 1;
+	    break;
+	  }
+    }
+  index_partial = partial;
+  
   /* If that failed, print an error. */
   if ((i < 0) || (!index_index[i]))
     {
-      info_error ((char *) _("No %sindex entries containing `%s'."),
-                  index_offset > 0 ? (char *) _("more ") : "", index_search);
+      info_error (index_offset > 0 ?
+		  _("No more index entries containing `%s'.") :
+		  _("No index entries containing `%s'."),
+		  index_search);
+      index_offset = 0;
       return;
     }
 
@@ -418,7 +478,7 @@ DECLARE_INFO_COMMAND (info_next_index_match,
       char *format;
 
       format = replace_in_documentation
-        ((char *) _("Found `%s' in %s. (`\\[next-index-match]' tries to find next.)"),
+        (_("Found `%s' in %s. (`\\[next-index-match]' tries to find next.)"),
          0);
 
       window_message_in_echo_area (format, match, (char *) name);
@@ -428,11 +488,12 @@ DECLARE_INFO_COMMAND (info_next_index_match,
   }
 
   /* Select the node corresponding to this index entry. */
-  node = info_get_node (index_index[i]->filename, index_index[i]->nodename);
+  node = info_get_node (index_index[i]->filename, index_index[i]->nodename,
+                        PARSE_NODE_DFLT);
 
   if (!node)
     {
-      info_error ((char *) msg_cant_file_node,
+      info_error (msg_cant_file_node,
                   index_index[i]->filename, index_index[i]->nodename);
       return;
     }
@@ -441,7 +502,8 @@ DECLARE_INFO_COMMAND (info_next_index_match,
 
   {
     long loc;
-    long line = index_index[i]->line_number - 1;
+    long line = window_log_to_phys_line (window,
+					 index_index[i]->line_number - 1);
 
     if (line >= 0 && line < window->line_count)
       {
@@ -452,7 +514,7 @@ DECLARE_INFO_COMMAND (info_next_index_match,
       {
 	/* Try to find an occurence of LABEL in this node. */
 	long start = window->line_starts[1] - window->node->contents;
-	loc = info_target_search_node (node, index_index[i]->label, start);
+	loc = info_target_search_node (node, index_index[i]->label, start, 1);
       }
 
     if (loc != -1)
@@ -475,11 +537,11 @@ REFERENCE **
 apropos_in_all_indices (char *search_string, int inform)
 {
   register int i, dir_index;
-  REFERENCE **all_indices = (REFERENCE **)NULL;
-  REFERENCE **dir_menu = (REFERENCE **)NULL;
+  REFERENCE **all_indices = NULL;
+  REFERENCE **dir_menu = NULL;
   NODE *dir_node;
 
-  dir_node = info_get_node ("dir", "Top");
+  dir_node = info_get_node ("dir", "Top", PARSE_NODE_DFLT);
   if (dir_node)
     dir_menu = info_menu_of_node (dir_node);
 
@@ -509,11 +571,12 @@ apropos_in_all_indices (char *search_string, int inform)
 
       /* Find this node.  If we cannot find it, try using the label of the
          entry as a file (i.e., "(LABEL)Top"). */
-      this_node = info_get_node (this_item->filename, this_item->nodename);
+      this_node = info_get_node (this_item->filename, this_item->nodename,
+                                 PARSE_NODE_VERBATIM);
 
       if (!this_node && this_item->nodename &&
           (strcmp (this_item->label, this_item->nodename) == 0))
-        this_node = info_get_node (this_item->label, "Top");
+        this_node = info_get_node (this_item->label, "Top", PARSE_NODE_DFLT);
 
       if (!this_node)
 	{
@@ -546,8 +609,7 @@ apropos_in_all_indices (char *search_string, int inform)
 	  }
 
         if (this_fb && inform)
-          message_in_echo_area ((char *) _("Scanning indices of `%s'..."),
-              files_name, NULL);
+          message_in_echo_area (_("Scanning indices of `%s'..."), files_name);
 
         this_index = info_indices_of_file_buffer (this_fb);
         free (this_node);
@@ -573,7 +635,7 @@ apropos_in_all_indices (char *search_string, int inform)
   /* Build a list of the references which contain SEARCH_STRING. */
   if (all_indices)
     {
-      REFERENCE *entry, **apropos_list = (REFERENCE **)NULL;
+      REFERENCE *entry, **apropos_list = NULL;
       int apropos_list_index = 0;
       int apropos_list_slots = 0;
 
@@ -597,7 +659,7 @@ apropos_in_all_indices (char *search_string, int inform)
       free (all_indices);
       all_indices = apropos_list;
     }
-  return (all_indices);
+  return all_indices;
 }
 
 #define APROPOS_NONE \
@@ -611,7 +673,7 @@ info_apropos (char *string)
   apropos_list = apropos_in_all_indices (string, 0);
 
   if (!apropos_list)
-    info_error ((char *) _(APROPOS_NONE), string, NULL);
+    info_error (_(APROPOS_NONE), string);
   else
     {
       register int i;
@@ -631,7 +693,7 @@ DECLARE_INFO_COMMAND (info_index_apropos,
 {
   char *line;
 
-  line = info_read_in_echo_area (window, (char *) _("Index apropos: "));
+  line = info_read_in_echo_area (window, _("Index apropos: "));
 
   window = active_window;
 
@@ -651,7 +713,7 @@ DECLARE_INFO_COMMAND (info_index_apropos,
       apropos_list = apropos_in_all_indices (line, 1);
 
       if (!apropos_list)
-        info_error ((char *) _(APROPOS_NONE), line, NULL);
+        info_error (_(APROPOS_NONE), line);
       else
         {
           register int i;
@@ -659,9 +721,9 @@ DECLARE_INFO_COMMAND (info_index_apropos,
 
           initialize_message_buffer ();
           printf_to_message_buffer
-            ((char *) _("\n* Menu: Nodes whose indices contain `%s':\n"),
-             line, NULL, NULL);
-          line_buffer = (char *)xmalloc (500);
+            (_("\n* Menu: Nodes whose indices contain `%s':\n"),
+             line);
+          line_buffer = xmalloc (500);
 
           for (i = 0; apropos_list[i]; i++)
             {
@@ -674,7 +736,7 @@ DECLARE_INFO_COMMAND (info_index_apropos,
               len = pad_to (40, line_buffer);
               sprintf (line_buffer + len, "(%s)%s.",
                        apropos_list[i]->filename, apropos_list[i]->nodename);
-              printf_to_message_buffer ("%s\n", line_buffer, NULL, NULL);
+              printf_to_message_buffer ("%s\n", line_buffer);
             }
           free (line_buffer);
         }
@@ -719,7 +781,7 @@ DECLARE_INFO_COMMAND (info_index_apropos,
 
             old_active = active_window;
             active_window = window;
-            new = window_make_window ((NODE *)NULL);
+            new = window_make_window (NULL);
             active_window = old_active;
           }
 
@@ -741,3 +803,160 @@ DECLARE_INFO_COMMAND (info_index_apropos,
   if (!info_error_was_printed)
     window_clear_echo_area ();
 }
+
+static FILE_BUFFER *
+create_virtindex_file_buffer (const char *filename, char *contents, size_t size)
+{
+  FILE_BUFFER *file_buffer;
+
+  file_buffer = make_file_buffer ();
+  file_buffer->filename = xstrdup (filename);
+  file_buffer->fullpath = xstrdup (filename);
+  file_buffer->finfo.st_size = 0;
+  file_buffer->flags = (N_IsInternal | N_CannotGC);
+
+  file_buffer->contents = contents;
+  file_buffer->filesize = size;
+  build_tags_and_nodes (file_buffer);
+  return file_buffer;
+}
+
+static NODE *
+create_virtindex_node (FILE_BUFFER *file_buffer)
+{
+  NODE *node;
+  TAG *tag = file_buffer->tags[0];
+  char *text = file_buffer->contents + tag->nodestart;
+
+  text += skip_node_separator (text);
+  
+  node = xmalloc (sizeof (NODE));
+  node->filename = file_buffer->filename;
+  node->nodename = xstrdup (tag->nodename);
+  node->contents = text;
+  node->nodelen = strlen (text);
+  node->body_start = strcspn(node->contents, "\n");
+
+  node->flags    = 0;
+  node->display_pos = 0;
+  node->parent = NULL;
+  node->flags = 0;
+  
+  return node;
+}
+
+#define NODECOL 41
+#define LINECOL 62
+
+static void
+format_reference (REFERENCE *ref, const char *filename, struct text_buffer *buf)
+{
+  size_t n;
+  
+  n = text_buffer_printf (buf, "* %s: ", ref->label);
+  if (n < NODECOL)
+    n += text_buffer_fill (buf, ' ', NODECOL - n);
+  
+  if (ref->filename && strcmp (ref->filename, filename))
+    n += text_buffer_printf (buf, "(%s)", ref->filename);
+  n += text_buffer_printf (buf, "%s. ", ref->nodename);
+
+  if (n < LINECOL)
+    n += text_buffer_fill (buf, ' ', LINECOL - n);
+  else
+    {
+      text_buffer_add_char (buf, '\n');
+      text_buffer_fill (buf, ' ', LINECOL);
+    }
+  
+  text_buffer_printf (buf, "(line %4d)\n", ref->line_number);
+}
+
+DECLARE_INFO_COMMAND (info_virtual_index,
+   _("List all matches of a string in the index"))
+{
+  char *line;
+  size_t linelen;
+  FILE_BUFFER *fb, *tfb;
+  NODE *node;
+  struct text_buffer text;
+  int i;
+  size_t cnt, off;
+  
+  fb = file_buffer_of_window (window);
+
+  if (!initial_index_filename ||
+      !fb ||
+      (FILENAME_CMP (initial_index_filename, fb->filename) != 0))
+    {
+      info_free_references (index_index);
+      window_message_in_echo_area (_("Finding index entries..."));
+      index_index = info_indices_of_file_buffer (fb);
+    }
+
+  if (!index_index)
+    {
+      info_error (_("No index"));
+      return;
+    }
+    
+  line = info_read_maybe_completing (window, _("Index topic: "),
+				     index_index);
+
+  /* User aborted? */
+  if (!line)
+    {
+      info_abort_key (window, 1, 1);
+      return;
+    }
+
+  if (mbslen (line) < min_search_length)
+    {
+      info_error (_("Search string too short"));
+      free (line);
+      return;
+    }
+  linelen = strlen (line);
+  
+  text_buffer_init (&text);
+  text_buffer_printf (&text, _("Index for `%s'"), line);
+  text_buffer_add_char (&text, 0);
+  off = text.off;
+  text_buffer_printf (&text,
+		      "\n\n%c\n%s %s,  %s %s,  %s Top\n\n"
+		      "Info Virtual Index\n"
+		      "******************\n\n"
+		      "Index entries that match `%s':\n\n"
+		      "* Menu:\n\n",
+		      INFO_COOKIE,
+		      INFO_FILE_LABEL, fb->filename,
+		      INFO_NODE_LABEL, text.base,
+		      INFO_UP_LABEL, line);
+  memmove (text.base, text.base + off, text.off - off);
+  text.off -= off;
+
+  cnt = 0;
+  for (i = 0; index_index[i]; i++)
+    {
+      if (string_in_line (line, index_index[i]->label) != -1)
+	{
+	  format_reference (index_index[i], fb->filename, &text);
+	  cnt++;
+	}
+    }
+
+  if (cnt == 0)
+    {
+      text_buffer_free (&text);
+      info_error (_("No index entries containing `%s'."), line);
+      return;
+    }
+
+  tfb = create_virtindex_file_buffer (fb->filename, text.base, text.off);
+  node = create_virtindex_node (tfb);
+  
+  info_set_node_of_window (1, window, node);
+  
+  if (!info_error_was_printed)
+    window_clear_echo_area ();
+}
