@@ -1,8 +1,8 @@
 /* infomap.c -- keymaps for Info.
-   $Id: infomap.c,v 1.18 2008/02/27 18:19:34 karl Exp $
+   $Id: infomap.c,v 1.27 2012/11/16 23:34:51 karl Exp $
 
-   Copyright (C) 1993, 1997, 1998, 1999, 2001, 2002, 2003, 2004, 2007, 2008
-   Free Software Foundation, Inc.
+   Copyright (C) 1993, 1997, 1998, 1999, 2001, 2002, 2003, 2004, 2007,
+   2008, 2011, 2012 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -45,7 +45,7 @@ keymap_make_keymap (void)
   for (i = 0; i < 256; i++)
     {
       keymap[i].type = ISFUNC;
-      keymap[i].function = (InfoCommand *)NULL;
+      keymap[i].function = NULL;
     }
 
   for (i = 'A'; i < ('Z' + 1); i++)
@@ -58,7 +58,7 @@ keymap_make_keymap (void)
       keymap[i].function = InfoCmd(info_do_lowercase_version);
     }
 
-  return (keymap);
+  return keymap;
 }
 
 #if defined(INFOKEY)
@@ -98,7 +98,7 @@ add_function_keyseq (InfoCommand *function,
       function == InfoCmd(info_do_lowercase_version) ||
       function == InfoCmd(ea_insert))
     return;
-  ks = (FUNCTION_KEYSEQ *)xmalloc (sizeof(FUNCTION_KEYSEQ));
+  ks = xmalloc (sizeof(FUNCTION_KEYSEQ));
   ks->next = function->keys;
   ks->map = rootmap;
   ks->keyseq = xstrdup(keyseq);
@@ -161,7 +161,7 @@ keymap_copy_keymap (Keymap map, Keymap rootmap, Keymap newroot)
           break;
         }
     }
-  return (keymap);
+  return keymap;
 }
 
 /* Free the keymap and its descendants. */
@@ -295,7 +295,6 @@ static unsigned char default_emacs_like_info_keys[] =
         CONTROL('b'), NUL,              A_info_backward_char,
         CONTROL('e'), NUL,              A_info_end_of_line,
         CONTROL('f'), NUL,              A_info_forward_char,
-        CONTROL('g'), NUL,              A_info_abort_key,
         CONTROL('h'), NUL,              A_info_get_help_window,
         CONTROL('l'), NUL,              A_info_redraw_display,
         CONTROL('n'), NUL,              A_info_next_line,
@@ -330,6 +329,7 @@ static unsigned char default_emacs_like_info_keys[] =
         'h', NUL,                       A_info_get_help_window,
         'H', NUL,                       A_info_get_info_help_node,
         'i', NUL,                       A_info_index_search,
+	'I', NUL,                       A_info_virtual_index,
         'l', NUL,                       A_info_history_node,
         'm', NUL,                       A_info_menu_item,
         'n', NUL,                       A_info_next_node,
@@ -405,22 +405,14 @@ static unsigned char default_emacs_like_info_keys[] =
         CONTROL('x'), 't', NUL,         A_info_tile_windows,
         CONTROL('x'), 'w', NUL,         A_info_toggle_wrap,
 
-        /* We want help to report q, not C-x C-c, etc.  */
-        'q', NUL,                       A_info_quit,
-        'x', NUL,                       A_info_delete_window,
-        SPC, NUL,                       A_info_scroll_forward,
-        DEL, NUL,                       A_info_scroll_backward,
-
 /*      Arrow key bindings for info keymaps.  It seems that some
         terminals do not match their termcap entries, so it's best to just
         define everything with both of the usual prefixes.  */
 
         SK_ESCAPE, SK_PAGE_UP, NUL,             A_info_scroll_backward,
         SK_ESCAPE, SK_PAGE_DOWN, NUL,           A_info_scroll_forward,
-        SK_ESCAPE, SK_UP_ARROW, NUL,            A_info_prev_line,
         '\033', 'O', 'A', NUL,                  A_info_prev_line,
         '\033', '[', 'A', NUL,                  A_info_prev_line,
-        SK_ESCAPE, SK_DOWN_ARROW, NUL,          A_info_next_line,
         '\033', 'O', 'B', NUL,                  A_info_next_line,
         '\033', '[', 'B', NUL,                  A_info_next_line,
         SK_ESCAPE, SK_RIGHT_ARROW, NUL,         A_info_forward_char,
@@ -447,8 +439,20 @@ static unsigned char default_emacs_like_info_keys[] =
         ESC, SK_ESCAPE, SK_LEFT_ARROW, NUL,     A_info_backward_word,
         ESC, '\033', 'O', 'D', NUL,             A_info_backward_word,
         ESC, '\033', '[', 'D', NUL,             A_info_backward_word,
+
+        /* We want help to report q, not C-x C-c, etc.  */
+        'q', NUL,                       A_info_quit,
+        'x', NUL,                       A_info_delete_window,
+        SPC, NUL,                       A_info_scroll_forward,
+        DEL, NUL,                       A_info_scroll_backward,
+        '{', NUL,                       A_info_search_previous,
+        '}', NUL,                       A_info_search_next,
+        CONTROL('g'), NUL,              A_info_abort_key,
+        SK_ESCAPE, SK_UP_ARROW, NUL,    A_info_prev_line,
+        SK_ESCAPE, SK_DOWN_ARROW, NUL,  A_info_next_line,
 };
 
+
 static unsigned char default_emacs_like_ea_keys[] =
 {
         0,      /* suppress-default-keybindings flag */
@@ -552,6 +556,7 @@ static unsigned char default_emacs_like_ea_keys[] =
         CONTROL('x'), SK_ESCAPE, SK_DELETE, NUL,A_ea_backward_kill_line,
 };
 
+
 static unsigned char default_vi_like_info_keys[] =
 {
         0,      /* suppress-default-keybindings flag */
@@ -574,7 +579,6 @@ static unsigned char default_vi_like_info_keys[] =
         CONTROL('d'), NUL,              A_info_scroll_half_screen_down,
         CONTROL('e'), NUL,              A_info_down_line,
         CONTROL('f'), NUL,              A_info_scroll_forward_page_only,
-        CONTROL('g'), NUL,              A_info_abort_key,
         CONTROL('k'), NUL,              A_info_up_line,
         CONTROL('l'), NUL,              A_info_redraw_display,
         CONTROL('n'), NUL,              A_info_down_line,
@@ -607,7 +611,7 @@ static unsigned char default_vi_like_info_keys[] =
         Meta('8'), NUL,                 A_info_menu_digit,
         Meta('9'), NUL,                 A_info_menu_digit,
         '<', NUL,                       A_info_first_node,
-        '>', NUL,                       A_info_last_node,
+	'>', NUL,                       A_info_last_node,
         '?', NUL,                       A_info_search_backward,
         '[', NUL,                       A_info_global_prev_node,
         ']', NUL,                       A_info_global_next_node,
@@ -661,6 +665,7 @@ static unsigned char default_vi_like_info_keys[] =
         ESC, 'f', NUL,                  A_info_xref_item,
         ESC, 'g', NUL,                  A_info_select_reference_this_line,
         ESC, 'h', NUL,                  A_info_get_info_help_node,
+	ESC, 'I', NUL,                  A_info_virtual_index,
         ESC, 'm', NUL,                  A_info_menu_item,
         ESC, 'n', NUL,                  A_info_search,
         ESC, 'N', NUL,                  A_info_search_backward,
@@ -699,22 +704,14 @@ static unsigned char default_vi_like_info_keys[] =
         CONTROL('x'), 'w', NUL,         A_info_toggle_wrap,
         CONTROL('x'), ',', NUL,         A_info_next_index_match,
 
-        /* We want help to report q, not C-x C-c, etc.  */
-        'q', NUL,                       A_info_quit,
-        'x', NUL,                       A_info_delete_window,
-        SPC, NUL,                       A_info_scroll_forward,
-        DEL, NUL,                       A_info_scroll_backward,
-
 /*      Arrow key bindings for info keymaps.  It seems that some
         terminals do not match their termcap entries, so it's best to just
         define everything with both of the usual prefixes.  */
 
         SK_ESCAPE, SK_PAGE_UP, NUL,             A_info_scroll_backward,
         SK_ESCAPE, SK_PAGE_DOWN, NUL,           A_info_scroll_forward,
-        SK_ESCAPE, SK_UP_ARROW, NUL,            A_info_up_line,
         '\033', 'O', 'A', NUL,                  A_info_up_line,
         '\033', '[', 'A', NUL,                  A_info_up_line,
-        SK_ESCAPE, SK_DOWN_ARROW, NUL,          A_info_down_line,
         '\033', 'O', 'B', NUL,                  A_info_down_line,
         '\033', '[', 'B', NUL,                  A_info_down_line,
         SK_ESCAPE, SK_RIGHT_ARROW, NUL,         A_info_scroll_forward_page_only,
@@ -741,8 +738,20 @@ static unsigned char default_vi_like_info_keys[] =
         ESC, '\033', 'O', 'D', NUL,             A_info_beginning_of_node,
         ESC, '\033', '[', 'D', NUL,             A_info_beginning_of_node,
         CONTROL('x'), SK_ESCAPE, SK_DELETE, NUL,A_ea_backward_kill_line,
+
+        /* We want help to report q, not C-x C-c, etc.  */
+        'q', NUL,                       A_info_quit,
+        'x', NUL,                       A_info_delete_window,
+        SPC, NUL,                       A_info_scroll_forward,
+        DEL, NUL,                       A_info_scroll_backward,
+        '{', NUL,                       A_info_search_previous,
+        '}', NUL,                       A_info_search_next,
+        CONTROL('g'), NUL,              A_info_abort_key,
+        SK_ESCAPE, SK_UP_ARROW, NUL,    A_info_up_line,
+        SK_ESCAPE, SK_DOWN_ARROW, NUL,  A_info_down_line,
 };
 
+
 static unsigned char default_vi_like_ea_keys[] =
 {
         0,      /* suppress-default-keybindings flag */
@@ -857,6 +866,7 @@ static unsigned char default_vi_like_ea_keys[] =
         CONTROL('x'), SK_ESCAPE, SK_DELETE, NUL,A_ea_backward_kill_line,
 };
 
+
 static unsigned char *user_info_keys;
 static unsigned int user_info_keys_len;
 static unsigned char *user_ea_keys;
@@ -920,7 +930,7 @@ fetch_user_maps (void)
                 strcat(filename, "/");
                 strcat(filename, INFOKEY_FILE);
         }
-#ifdef __MSDOS__
+#if defined(__MSDOS__) || defined(__MINGW32__)
         /* Poor baby, she doesn't have a HOME...  */
         else
                 filename = xstrdup(INFOKEY_FILE); /* try current directory */
@@ -929,8 +939,7 @@ fetch_user_maps (void)
         {
                 if (filename && errno != ENOENT)
                 {
-                        info_error(filesys_error_string(filename, errno),
-                            NULL, NULL);
+                        info_error("%s", filesys_error_string(filename, errno));
                         free(filename);
                 }
                 return 0;
@@ -945,23 +954,23 @@ fetch_user_maps (void)
                    more than 100 KB is a problem). */
                 if (len < INFOKEY_NMAGIC + 2)
                         info_error(_("Ignoring invalid infokey file `%s' - too small"),
-                                   filename, NULL);
+                                   filename);
                 else
                         info_error(_("Ignoring invalid infokey file `%s' - too big"),
-                                   filename, NULL);
+                                   filename);
                 close(f);
                 free(filename);
                 return 0;
         }
 
         /* Read the file into a buffer. */
-        buf = (unsigned char *)xmalloc((int)len);
+        buf = xmalloc((int)len);
         nread = read(f, buf, (unsigned int) len);
         close(f);
         if ((unsigned int) nread != len)
         {
                 info_error(_("Error reading infokey file `%s' - short read"),
-                    filename, NULL);
+                           filename);
                 free(buf);
                 free(filename);
                 return 0;
@@ -980,7 +989,7 @@ fetch_user_maps (void)
         )
         {
                 info_error(_("Invalid infokey file `%s' (bad magic numbers) -- run infokey to update it"),
-                    filename, NULL);
+                           filename);
                 free(filename);
                 return 0;
         }
@@ -989,7 +998,7 @@ fetch_user_maps (void)
         {
                 info_error
                   (_("Your infokey file `%s' is out of date -- run infokey to update it"),
-                    filename, NULL);
+                    filename);
                 free(filename);
                 return 0;
         }
@@ -1005,7 +1014,7 @@ fetch_user_maps (void)
                 if (n < 0 || (unsigned int) n > len - 4 - (p - buf))
                 {
                         info_error(_("Invalid infokey file `%s' (bad section length) -- run infokey to update it"),
-                            filename, NULL);
+                            filename);
                         free(filename);
                         return 0;
                 }
@@ -1026,7 +1035,7 @@ fetch_user_maps (void)
                         break;
                 default:
                         info_error(_("Invalid infokey file `%s' (bad section code) -- run infokey to update it"),
-                            filename, NULL);
+                            filename);
                         free(filename);
                         return 0;
                 }
@@ -1160,8 +1169,7 @@ section_to_keymaps(Keymap map, unsigned char *table, unsigned int len)
                 }
         }
         if (state != getseq)
-                info_error(_("Bad data in infokey file -- some key bindings ignored"),
-                    NULL, NULL);
+                info_error("%s", _("Bad data in infokey file -- some key bindings ignored"));
         return !stop;
 }
 
@@ -1210,8 +1218,7 @@ section_to_vars(unsigned char *table, unsigned int len)
               }
           }
       if (state != getvar)
-        info_error(_("Bad data in infokey file -- some var settings ignored"),
-            NULL, NULL);
+        info_error("%s", _("Bad data in infokey file -- some var settings ignored"));
 }
 
 void
